@@ -103,6 +103,15 @@ struct ct_data {
 };
 
 
+struct mimestate {
+	int	lastch;
+	int	lastwasnl;
+	int	convertmode;
+	int	column;
+	int	alarmcnt;
+};
+
+
 #ifndef __
 # ifndef __STDC__
 #  define __(x) ()
