@@ -12,12 +12,12 @@
 
 /* send a signal to an already-running daemon process or process group */
 
-void
+int
 killprevious(sig, pidfile)
 	int sig;
 	char *pidfile;
 {
-	int c;
+	int c, rc;
 	FILE *fp;
 	char *path, buf[128];
 	extern char *postoffice;
@@ -40,11 +40,17 @@ killprevious(sig, pidfile)
 		}
 		fclose(fp);
 	}
+	rc = 0;
 	if ((sig == 0 || sig == SIGTERM || -sig == SIGTERM)
 	    && (fp = fopen(path, "w+")) != NULL) {
 		setvbuf(fp, buf, _IOFBF, sizeof buf);
 		fprintf(fp, "%d\n", getpid());
-		fclose(fp);
+		rc = fflush(fp);
+		if (fclose(fp) != 0)
+		  rc |= 1; /* indicate error */
+		if (rc != 0)
+		  unlink(path);
 	}
 	free(path);
+	return rc;
 }
