@@ -48,7 +48,7 @@ initline(blksize)
 void
 repos_getline(fp,newpos)
 	FILE *fp;
-	long newpos;
+	off_t newpos;
 {
 	getline_blen = 0;
 	fseek(fp, newpos, 0);
