@@ -2545,11 +2545,12 @@ fapply(shcmdp, l)
 {
 	register struct conscell *ll;
 	int argc = 0;
-	char *argv[20];	/* XX: argc never to exceed magic number */
+#define FARGCMAX 30
+	char *argv[FARGCMAX];	/* XX: argc never to exceed magic number */
 
 	if (shcmdp->sptr != NULL) {
 		argv[argc++] = shcmdp->name;
-		for (ll = car(l); ll != NULL; ll = cdr(ll)) {
+		for (ll = car(l); ll != NULL && argc < FARGCMAX-1; ll = cdr(ll)) {
 			if (STRING(ll))
 				argv[argc++] = (char *)ll->string;
 		}
