@@ -225,7 +225,7 @@ v_expand(s, caller, retcode)
 			fprintf(stderr,
 				"%s: BAD magic retcode on $? expansion!\n",
 				progname);
-			abort();
+			abort(); /* Bad magic retcode on $? expansion! */
 		}
 		sprintf(np, "%d", retcode);
 		return newstring((u_char *)strsave(np));
