@@ -1,4 +1,4 @@
-# !/bin/sh -
+#! /bin/sh -
 
 ZCONFIG=/etc/zmailer.conf
 . $ZCONFIG
@@ -18,7 +18,7 @@ if [ -s /tmp/cancel$$ ]; then
 Subject: cancel message(s) $@
 Cc: postmaster" ; sed -e 's/^/Obsoletes: /' < /tmp/cancel$$ ; echo '
 
-') | /usr/lib/sendmail -t
+') | $SENDMAILPATH -t
 	echo "Submitted cancel message for:
 "
 	sed -e 's/^/	/' < /tmp/cancel$$
