@@ -91,7 +91,7 @@ extern void initline();
 extern void setfreefd();
 extern void v_purge();
 
-extern char *newattribute_2 __((struct conscell *, char *, char*));
+extern char *newattribute_2 __((char *, char *, char*));
 
 #ifndef strchr
 extern char *strchr(), *strrchr();
@@ -657,7 +657,7 @@ rd_doit(filename, dirs)
 	av[0] = "process"; /* I think this needs to be here */
 	av[1] = filename;
 	av[2] = NULL;
-	s_apply(2, &av[0]);
+	s_apply(2, &av[0]); /* "process" filename (within  rd_doit() ) */
 	free_gensym();
 
 	setlevel(MEM_SHCMD,sh_memlevel);
@@ -1000,7 +1000,7 @@ run_process(argc, argv)
 		if (strmatch(pswp->pattern, file)) {
 			printf("process %s %s\n", pswp->function, file);
 			argv[0] = pswp->function;
-			r = s_apply(argc, argv);
+			r = s_apply(argc, argv); /* process-by-FUNC filename */
 			printf("done with %s\n", file);
 			if (r)
 				printf("status %d\n", r);
@@ -1256,9 +1256,8 @@ run_listexpand(avl, il)
 	struct address *ap;
 	struct token **pt, *t;
 	struct addr *pp;
-	struct conscell *attributes, *tmp;
-	struct conscell *al, *alp = NULL;
-	char *localpart, *origaddr;
+	struct conscell *al, *alp = NULL, *tmp;
+	char *localpart, *origaddr, *attributenam;
 	int c, n, len, errflag, stuff;
 	volatile int cnt;
 	char *comment, *erroraddress;
@@ -1272,6 +1271,7 @@ run_listexpand(avl, il)
 	char *olderrors = errors_to;
 	volatile FILE *errstr = stderr;
 	char *notary = NULL;
+	int no_dsn = 0;
 
 	extern struct conscell *s_value;
 
@@ -1313,6 +1313,8 @@ run_listexpand(avl, il)
 	      if (strchr(notary,'\n') != NULL ||
 		  strchr(notary,'\r') != NULL)
 		errflag = 1;
+	      if (strcmp(notary,"-")==0)
+		no_dsn = 1;
 	      break;
 	    default:
 	      errflag = 1;
@@ -1327,7 +1329,8 @@ run_listexpand(avl, il)
 	tmp = il;
 	for (; tmp != NULL; tmp = cdr(tmp)) ++cnt; /* Count arguments.. */
 
-	if (errflag || cnt != 3) {
+	if (errflag || cnt != 3 ||
+	    !STRING(il) || !STRING(cdr(il)) || !STRING(cddr(il)) ) {
 		fprintf(stderr,
 			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [-p privilege] [ -c comment ] [ -N notarystring ] $attribute $localpart $origaddr < /file/path \n",
 		car(avl)->string);
@@ -1337,11 +1340,9 @@ run_listexpand(avl, il)
 		return NULL;
 	}
 
-	attributes = copycell(il);
-	car(attributes) = s_copy_tree(car(attributes));
-	cdr(attributes) = NULL;
-	localpart  = (char*) cdr(il)->string;
-	origaddr   = (char*) cddr(il)->string;
+	attributenam = (char*)    (il)->string;
+	localpart    = (char*) cdr(il)->string;
+	origaddr     = (char*) cddr(il)->string;
 
 	/* We (memory-)leak this stuff for a moment.. */
 	e = (struct envelope *)tmalloc(sizeof (struct envelope));
@@ -1485,6 +1486,7 @@ run_listexpand(avl, il)
 	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
 		char *av[5];
 		int rc;
+		int omem;
 		struct conscell *l, *d, *lrc;
 		char *s, *s2, *se;
 
@@ -1497,45 +1499,58 @@ run_listexpand(avl, il)
 
 #define use_lapply 1
 #ifdef use_lapply
-		*DSNbuf = 0;
-		if (notary != NULL)
-		  strncpy(DSNbuf,notary,sizeof(DSNbuf)-30);
-		DSNbuf[sizeof(DSNbuf)-30] = 0; /* Be brutal, and chop
-						  off the tail, if it is
-						  too long.. */
-		s2 = strlen(DSNbuf)+DSNbuf;
-		if (s2 > DSNbuf)
-		  strcpy(s2++," ");
-		strcpy(s2,"ORCPT=rfc822;");
-		s2 += strlen(s2);
-		se = DSNbuf + sizeof(DSNbuf)-1; /* BUF-end */
-		s = buf;
-		while (*s) {
-		  u_char c = *s;
-		  if ('!' <= c && c <= '~' && c != '+' && c != '=') {
-		    if (s2 < se)
-		      *s2++ = c;
-		  } else if (s2 < se) {
-		    sprintf(s2,"+%02X",c);
-		    s2 += 3;
+		if (!no_dsn) {
+		  *DSNbuf = 0;
+		  if (notary != NULL)
+		    strncpy(DSNbuf,notary,sizeof(DSNbuf)-30);
+		  DSNbuf[sizeof(DSNbuf)-30] = 0; /* Be brutal, and chop
+						    off the tail, if it is
+						    too long.. */
+		  s2 = strlen(DSNbuf)+DSNbuf;
+		  if (s2 > DSNbuf)
+		    strcpy(s2++," ");
+		  strcpy(s2,"ORCPT=rfc822;");
+		  s2 += strlen(s2);
+		  se = DSNbuf + sizeof(DSNbuf)-1; /* BUF-end */
+		  s = buf;
+		  while (*s) {
+		    u_char c = *s;
+		    if ('!' <= c && c <= '~' && c != '+' && c != '=') {
+		      if (s2 < se)
+			*s2++ = c;
+		    } else if (s2 < se) {
+		      sprintf(s2,"+%02X",c);
+		      s2 += 3;
+		    }
+		    ++s;
 		  }
-		  ++s;
+		  *s2 = 0;
+		  s = newattribute_2(attributenam,"DSN",DSNbuf);
+		} else {
+		  s = attributenam;
 		}
-		*s2 = 0;
-		s = newattribute_2(attributes,"DSN",DSNbuf);
+
+		omem = stickymem;
+		/* stickymem = MEM_MALLOC; */
 
 		l       = newstring((u_char*)strsave(buf));
-		cdr(l)  = conststring((u_char*)origaddr);
+		cdr(l)  = newstring((u_char*)strsave(origaddr));
 		cddr(l) = newstring((u_char*)strsave(s));
 		l = ncons(l);
 
+		stickymem = omem;
+
 		deferit = 0;
 		v_set(DEFER, "");
 
 		rc = l_apply("rrouter", l);
 		lrc = s_value;
 		s_value = NULL;
-		s_free_tree(l); /* We can clean up the input list */
+
+		omem = stickymem;
+		/* stickymem = MEM_MALLOC;
+		   s_free_tree(l); */ /* We can clean up the input list */
+		stickymem = omem;
 #else
 		lrc = router(ap, privilege, "recipient");
 		rc = (lrc == NULL);
@@ -2013,7 +2028,7 @@ run_runas(argc, argv)
 		}
 	}
 	/* must be builtin or defined function */
-	r = s_apply(argc - 2, argv + 2);
+	r = s_apply(argc - 2, argv + 2); /* within:  run_runas() */
 	if (myuid == 0 && uid != 0) {
 	  if (setreuid(0, 0) < 0)
 	    abort(); /* user-identity change failed! */
