@@ -33,11 +33,6 @@
 #include <resolv.h>
 #include "search.h"
 
-#ifdef	__alpha		/* The IP-address processing must be changed to
-			   something truly portable, this is awful.. */
-# define u_long u_int
-#endif
-
 /*
  * Query a BIND standard (version 4.7 or later) nameserver.
  */
@@ -385,12 +380,12 @@ getmxrr(host, localhost, ttlp, depth)
 			break;
 		cp += n;
 		type = _getshort(cp);
- 		cp += sizeof(u_short);				/* type */
- 		cp += sizeof(u_short);				/* class */
+ 		cp += 2;				/* type -- short */
+ 		cp += 2;				/* class -- short */
 		ttl = (time_t) _getlong(cp);
-		cp += sizeof(u_long);				/* ttl */
+		cp += 4;				/* ttl -- "long" */
 		n = _getshort(cp);
-		cp += sizeof(u_short);				/* dlen */
+		cp += 2;				/* dlen -- short */
 		if (type == T_CNAME) {
 			cp += dn_expand((msgdata *)&answer, eom, cp,
 					(void*)realname, sizeof realname);
@@ -405,7 +400,7 @@ getmxrr(host, localhost, ttlp, depth)
 		}
 		mx[nmx].ttl = ttl;
 		mx[nmx].pref = _getshort(cp);
-		cp += sizeof(u_short);		/* MX preference value */
+		cp += 2; /* "short" */		/* MX preference value */
 		n = dn_expand((msgdata *)&answer, eom, cp, (void*)hbuf, sizeof hbuf);
 		if (n < 0)
 			break;
@@ -602,7 +597,8 @@ getrrtype(host, rrtype, ttlp, depth)
 	maxttl = 0;
 	l = NULL;
 	for (lhead = NULL; --ancount >= 0 && cp < eom; cp = nextcp) {
-		n = dn_expand((msgdata *)&answer, eom, cp, (void*)nbuf, sizeof(nbuf));
+		n = dn_expand((msgdata *)&answer, eom, cp,
+			      (void*)nbuf, sizeof(nbuf));
 		if (n < 0)
 			break;
 		if (first) {
@@ -616,12 +612,12 @@ getrrtype(host, rrtype, ttlp, depth)
 		}
 		cp += n;
 		type = _getshort(cp);
- 		cp += sizeof(u_short);				/* type */
- 		cp += sizeof(u_short);				/* class */
+ 		cp += 2;			/* type  -- short */
+ 		cp += 2;			/* class -- short */
 		ttl = (time_t) _getlong(cp);
-		cp += sizeof(u_long);				/* ttl */
+		cp += 4;			/* ttl -- "long" */
 		n = _getshort(cp);
-		cp += sizeof(u_short);				/* dlen */
+		cp += 2;			/* dlen -- short */
 		nextcp = cp + n;
 
 		if (rrtype != T_ANY && type != rrtype)
@@ -658,10 +654,10 @@ getrrtype(host, rrtype, ttlp, depth)
 			if (rrtype != T_WKS)	/* Take it only if it was */
 			  continue;		/* explicitely asked for! */
 
-			if (n < sizeof (u_long) + 1)
+			if (n < (4/*"long"*/ + 1))
 				continue;
 			ok = 0;
-			cp += sizeof(u_long);		/* skip IP address */
+			cp += 4;		/* skip IP address */
 			if (*cp++ == IPPROTO_TCP) {	/* check protocol */
 			  if (cp + (IPPORT_SMTP/8) < nextcp &&
 			      (*(cp+(IPPORT_SMTP/8)) &
