@@ -317,6 +317,8 @@ modp_gdbm(sip)
 				sip->file);
 		return 0;
 	}
+	if (stbuf.st_nlink == 0)
+		return 1;	/* Unlinked underneath of us! */
 
 	symid = symbol((u_char *)sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
