@@ -76,7 +76,7 @@ reopen:
 					   FILENO(fp), 0);
 		else
 		  fm->membuf = NULL;
-		if ((long)fm->membuf == -1) {
+		if ((int)fm->membuf == -1) {
 			fprintf(stderr,
 				"search_bin: cannot mmap() (r/o) %ld bytes of file \"%s\" into memory, errno=%d",
 				(long)fst.st_size, sip->file, errno);
@@ -121,8 +121,9 @@ reopen:
 	  int linecnt = 0;
 	  char *buf = fm->membuf;
 	  char *eof = fm->membuf + fm->size;
-	  while (buf < eof)
-	    if (*++buf == '\n') ++linecnt;
+	  for (;buf < eof; ++buf)
+	    if (*buf == '\n')
+	      ++linecnt;
 	  buf = fm->membuf;
 	  fm->offsets = (int*)emalloc(sizeof(int)*(linecnt+1));
 	  linecnt = 0;
