@@ -384,6 +384,9 @@ modp_seq(sip)
 		return 0;
 	}
 
+	if (stbuf.st_nlink == 0) /* Changed underneath of us! */
+		return 1;
+
 	symid = symbol((u_char *)sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
