@@ -293,7 +293,7 @@ main(argc, argv)
 #ifdef	XMEM
 write(30, "\n", 1);
 #endif	/* XMEM */
-	    s_apply(2, &av[0]);
+	    s_apply(2, &av[0]); /* "process" filename */
 	  } while (++c < argc);
 	} else if (daemonflg) {
 	  av[0] = "daemon";
