@@ -977,7 +977,7 @@ report(fp)
 	  if (onlyuser)
 	    printf("No user messages found\n");
 	  else
-	    printf("Transport queue is empty\n");
+	    printf("Transport queue is empty -- or scheduler uses -Q -mode\n");
 	}
 }
 
