@@ -99,7 +99,8 @@ scnotaryreport(errfp,notary,haserrsp)
 	} else {
 	  *diagnostic++ = 0;
 	  wtt      = strchr(diagnostic,'\001');
-	  *wtt++    = 0;
+	  if (wtt) /* Uhh... Mal-formed input from the transporters ? */
+	    *wtt++    = 0;
 	}
 
 	if (*rcpt == '\003' /* XX: MAGIC! */) {
@@ -439,7 +440,7 @@ reporterrs(cfpi)
 	    sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
 		    deliveryform ? deliveryform : "delivery");
 	    if ((fp = fopen(path, "r")) != NULL) {
-	      int inhdr = 1;
+	      int inhdr = 1, hadsubj =0;
 	      buf[sizeof(buf)-1] = 0;
 	      while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
 		if (strncmp(buf,"HDR",3)==0) {
@@ -466,7 +467,20 @@ reporterrs(cfpi)
 		} else if (strncmp(buf,"ADR",3)==0) {
 		  fputs(buf+4,errfp);
 		} else if (strncmp(buf,"SUB",3)==0) {
-		  fputs(buf+4,errfp);
+		  hadsubj = 1;
+		  if (no_error_report < 0) {
+		    /* We modify the subject string... */
+		    char *s = buf+4;
+		    while (*s && *s != ':') ++s;
+		    if (*s == ':') {
+		      ++s;
+		      if (*s) *s++ = 0;
+		      fprintf(errfp,"%s Double-fault: %s", buf+4, s);
+		    } else {
+		      fputs("Subject: Double-fault: unknown delivery error\n",errfp);
+		    }
+		  } else
+		    fputs(buf+4,errfp);
 		} else {
 		  if (inhdr) {
 		    inhdr = 0;
