@@ -29,6 +29,8 @@ static struct threadgroup *thrg_root   = NULL;
 int idleprocs = 0;
 extern int global_wrkcnt;
 extern int syncstart;
+extern int freeze;
+extern int slow_shutdown;
 extern time_t now;
 extern char *procselect, *procselhost;
 
@@ -638,7 +640,8 @@ struct thread *thr;
 	/* 4) re-arrange pointers */
 	if (thr->thgrp->ce.flags & CFG_AGEORDER) {
 	  /* ctime order */
-	  qsort((void*)ur_arr, n, sizeof(struct vertex *), vtx_ctime_cmp);
+	  if (n > 1)
+	    qsort((void*)ur_arr, n, sizeof(struct vertex *), vtx_ctime_cmp);
 	} else
 	  /* Random order */
 	  for (i = 0; i < n; ++i) {
@@ -687,7 +690,7 @@ struct thread *thr;
 	struct web         *ch  = vp->orig[L_CHANNEL];
 
 
-	if (syncstart) return 0;
+	if (syncstart || (freeze && !slow_shutdown)) return 0;
 	if (procselect) {
 	  if (*procselect != '*' &&
 	      strcmp(procselect,ch->name) != 0)
