@@ -26,6 +26,7 @@
 #include "prototypes.h"
 
 extern int forkrate_limit;
+extern int freeze;
 
 static int  scheduler_nofiles = -1; /* Will be filled below */
 static int  runcommand __((char **, struct vertex *, struct web *, struct web*));
@@ -61,10 +62,13 @@ struct procinfo *cpids = NULL;
 #define	MAXFILESPERTRANSPORT	1000
 
 int	numkids = 0;
+int	readsockcnt = 0; /* Count how many childs to read there are;
+			    this for the SLOW Shutdown */
 
 extern char *getzenv();
 extern int errno;
 
+extern int slow_shutdown;
 
 /* Send "#idle\n" -string to the child.. */
 
@@ -82,8 +86,13 @@ struct procinfo *proc;
 	  /* abort(); */ /* Hmm.. ??? */
 	  return;
 	}
-	strcpy(proc->cmdbuf,"#idle\n");
-	len = strlen(proc->cmdbuf);
+	if (slow_shutdown) {
+	  proc->cmdbuf[0] = '\n';
+	  len = 1;
+	} else {
+	  strcpy(proc->cmdbuf,"#idle\n");
+	  len = strlen(proc->cmdbuf);
+	}
 	rc = write(proc->tofd, proc->cmdbuf, len);
 	if (rc != len) {
 	  if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
@@ -187,7 +196,9 @@ struct procinfo *proc;
 	  return;
 	}
 
-	if (vtx->cfp->dirind > 0) {
+	if (slow_shutdown)
+	  strcpy(cmdbuf,"\n");
+	else if (vtx->cfp->dirind > 0) {
 	  char *d = cfpdirname(vtx->cfp->dirind);
 	  if (proc->thg->withhost)
 	    sprintf(cmdbuf, "%s/%s\n", d, vtx->cfp->mid);
@@ -253,6 +264,8 @@ start_child(vhead, chwp, howp)
 	time_t this_time;
 
 
+	if (freeze) return 0;
+
 	if (verbose)
 	  printf("transport(vhead,chan=%s,host=%s)\n",
 		 chwp->name,howp->name);
@@ -435,6 +448,28 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  fd_nonblockingmode(tofd);
 }
 
+/*
+ * shutdown all kids that we have
+ */
+void
+shutdown_kids()
+{
+	int i;
+	struct procinfo *proc = cpids;
+
+	if (!cpids) return; /* Nothing to do! */
+
+	for (i = 0; i < scheduler_nofiles; ++i,++proc)
+	  if (proc->pid > 0 && proc->tofd >= 0) {
+	    /* Send the death-marker to the kid, and
+	       then close the command channel */
+	    kill(SIGQUIT, proc->pid);
+	    write(proc->tofd,"\n\n",2);
+	    pipes_shutdown_child(proc->tofd);
+	    proc->tofd = -1;
+	  }
+}
+
 /* 
  *  Reclaim the process slot -- this process is dead now.
  */
@@ -608,6 +643,7 @@ time_t timeout;
 	fd_set	rdmask;
 	fd_set	wrmask;
 	struct timeval tv;
+	struct procinfo *proc = cpids;
 
 	if (in_select) {
 	  fprintf(stderr,"**** recursed into mux()! ***\n");
@@ -624,12 +660,14 @@ time_t timeout;
 	maxf = -1;
 	_FD_ZERO(rdmask);
 	_FD_ZERO(wrmask);
+	readsockcnt = 0;
 	if (cpids != NULL)
-	  for (i = 0; i < scheduler_nofiles ; ++i)
-	    if (cpids[i].pid != 0) {
+	  for (proc = cpids,i = 0; i < scheduler_nofiles ; ++i,++proc)
+	    if (proc->pid != 0) {
 	      _FD_SET(i, rdmask);
 	      maxf = i;
-	      if (cpids[i].cmdlen != 0)
+	      ++readsockcnt;
+	      if (proc->cmdlen != 0)
 		_FD_SET(i, wrmask);
 	    }
 	if (querysocket >= 0) {
@@ -655,11 +693,9 @@ time_t timeout;
 	  if (errno == EINVAL || errno == EBADF) {
 	    fprintf(stderr, "** select() returned errno=%d\n", err);
 	    for (i = 0; i < maxf; ++i) {
-	      if (_FD_ISSET(i,rdmask) &&
-		  fcntl(i,F_GETFL,0) < 0)
+	      if (_FD_ISSET(i,rdmask)  &&  fcntl(i,F_GETFL,0) < 0)
 		fprintf(stderr,"** Invalid fd on a select() rdmask: %d\n",i);
-	      if (_FD_ISSET(i,wrmask) &&
-		  fcntl(i,F_GETFL,0) < 0)
+	      if (_FD_ISSET(i,wrmask)  &&  fcntl(i,F_GETFL,0) < 0)
 		fprintf(stderr,"** Invalid fd on a select() wrmask: %d\n",i);
 	    }
 	    fflush(stderr);
@@ -689,9 +725,19 @@ time_t timeout;
 	      rc = fork();
 	      if (rc == 0) { /* Child! */
 		close(querysocket);
+#ifdef HAVE_TCPD_H /* TCP-Wrapper code */
+		if (wantconn(i, "mailq") == 0) {
+		  char *msg = "refusing 'mailq' query from your whereabouts\r\n";
+		  int   len = strlen(msg);
+		  write(i,msg,len);
+		  _exit(0);
+		}
+#endif
 		qprint(i);
-		execl("/bin/false",NULL);
-		_exit(0);
+		/* Silence memory debuggers about this child's
+		   activities by doing exec() on the process.. */
+		/* execl("/bin/false","false",NULL); */
+		_exit(0); /* _exit() should be silent too.. */
 	      }
 	      /* if (rc > 0)
 		 ++numkids; */
@@ -750,9 +796,19 @@ queryipccheck()
 	    if (n >= 0) {
 	      int pid = fork();
 	      if (pid == 0) {
+#ifdef HAVE_TCPD_H /* TCP-Wrapper code */
+		if (wantconn(n, "mailq") == 0) {
+		  char *msg = "refusing 'mailq' query from your whereabouts\r\n";
+		  int   len = strlen(msg);
+		  write(n,msg,len);
+		  _exit(0);
+		}
+#endif
 		qprint(n);
-		execl("/bin/false",NULL);
-		_exit(0);
+		/* Silence memory debuggers about this child's
+		   activities by doing exec() on the process.. */
+		/* execl("/bin/false","false",NULL); */
+		_exit(0); /* _exit() should be silent too.. */
 	      }
 	      close(n);
 	    }
@@ -816,10 +872,13 @@ time_t timeout;
 	 * are possible in the absence of async i/o like select() or the
 	 * simulation USG supplies, but it ain't worth the hassle.
 	 */
+	readsockcnt = 0;
 	if (cpids != NULL)
 	  for (fd = 0; fd < scheduler_nofiles ; ++fd)
-	    if (cpids[fd].pid != 0)
+	    if (cpids[fd].pid != 0) {
 	      readfrom(fd);
+	      ++readsockcnt;
+	    }
 
 	mytime(&now);
 	if (timeout > now)
