@@ -30,6 +30,7 @@ extern int eunlink();
 extern struct spblk *sp_fhead(), *sp_fnext();
 extern time_t now;
 extern int global_wrkcnt;
+extern char *procselect;
 
 /*
  * Parse a diagnostic string from a transport agent, and do whatever is
@@ -232,7 +233,7 @@ unctlfile(cfp, no_unlink)
 	else
 	  strcpy(path, cfp->mid);
 
-	if (!no_unlink) {
+	if (!no_unlink && !procselect) {
 	  reporterrs(cfp);
 
 	  if (do_syslog)
@@ -738,7 +739,7 @@ static int u_error(vp, index, inum, offset, notary, message)
 		"%s %s: %ld/%ld/%s/error %s\n", timestring(),
 		vp->thgrp->ce.command, inum, offset, notary, message);
 
-	if (vp->notaryflg & NOT_FAILURE)
+	if (!procselect && vp->notaryflg & NOT_FAILURE)
 	  msgerror(vp, offset, message);
 
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
