@@ -163,6 +163,18 @@ struct smtpconf {
 	struct smtpconf *next;
 };
 
+#include "relaytest.h"
+
+struct relaytest *relaytargets    = NULL;
+struct relaytest *relaysourcenets = NULL;
+struct relaytest *relaysourcedoms = NULL;
+int accept_sourcenet = 1, accept_sourcedomain = 1;
+
+struct relaytest *rejectnets     = NULL;
+struct relaytest *rejectsource   = NULL;
+struct relaytest *rejecttarget   = NULL;
+int reject_net = 0, reject_source = 0;
+
 extern char *rfc821_domain __((char *s, int strict));
 extern char *rfc821_path   __((char *s, int strict));
 extern char *rfc821_path2  __((char *s, int strict));
@@ -180,6 +192,7 @@ char	myhostname[MAXHOSTNAMELEN+1];
 char	rhostname[MAXHOSTNAMELEN+1];
 char	ihostaddr[3*4+1*3+2+1 /* == 18 for '[255.255.255.255]' */ +10/*slack*/];
 int	rport;
+int	debug	   = 0;
 int	skeptical  = 1;
 int	checkhelo  = 0;
 int	verbose    = 0;
@@ -190,6 +203,8 @@ int	pid, routerpid;
 FILE	*logfp = NULL;
 int	D_alloc = 0;
 
+char *helplines[20] = { NULL, };
+
 static char *m200 = "2.0.0";
 static char *m400 = "4.0.0";
 static char *m430 = "4.3.0";
@@ -224,6 +239,7 @@ FILE *mfp = NULL;
 struct smtpconf *cfinfo = NULL;
 long availspace = -1;	/* available diskspace/2 in bytes	*/
 long maxsize    = 0;
+int  percent_accept = 0;
 long sizeoptval = 0;	/* "MAIL FROM:<xxx> SIZE=nnn" -value	*/
 long sizeoptsum = 0;
 int maxloadavg = 999;	/* Maximum load-average that is tolerated
@@ -250,7 +266,7 @@ extern int getpeername();
 static RETSIGTYPE reaper __((int sig));
 static RETSIGTYPE timedout __((int sig));
 extern void detach __((void)), prversion __(());
-static void smtpserver __((int insecure));
+static void smtpserver __((int insecure, struct sockaddr_in *));
 static void setrhostname __((struct sockaddr_in *));
 extern int  killprevious __(());
 extern void settrusteduser __(());
@@ -368,7 +384,7 @@ main(argc, argv)
 	  exit(1);
 	}
 	/* optarg = NULL; */
-	while ((c = getopt(argc, argv, "aBighl:np:L:M:P:R:s:Vv")) != EOF) {
+	while ((c = getopt(argc, argv, "aBd:ighl:np:L:M:P:R:s:Vv")) != EOF) {
 	  switch (c) {
 	    case 'a':
 		ident_flag = 1;
@@ -376,6 +392,9 @@ main(argc, argv)
 	    case 'B':
 		with_protocol = WITH_BSMTP;
 		break;
+	    case 'd':
+		debug = atoi(optarg);
+		break;
 	    case 'M':
 		maxsize = atoi(optarg);
 		if (maxsize < 0)
@@ -473,11 +492,12 @@ main(argc, argv)
 	  strcpy(rhostname, "stdin");
 	  ihostaddr[0] = '\0';
 	  sprintf(ident_username, "uid#%d@localhost", getuid());
-	  smtpserver(0);
+	  smtpserver(0,NULL);
 	} else
 #ifdef	USE_INET
 	  if (inetd) {
-	    if (maxloadavg < loadavg_current()) {
+	    if (maxloadavg != 999 &&
+		maxloadavg < loadavg_current()) {
 	      write(1, msg_toohighload, strlen(msg_toohighload));
 	      exit(1);
 	    }
@@ -489,7 +509,7 @@ main(argc, argv)
 	      exit(1);
 	    }
 	    setrhostname(&raddr);
-	    openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
+	    openlog("smtpserver", LOG_PID, LOG_MAIL);
 
 	    if (ident_flag != 0)
 	      setrfc1413ident(0,&raddr);
@@ -505,7 +525,6 @@ main(argc, argv)
 	    if (logfp != NULL)
 	      fprintf(logfp, "%d#\tconnection from %s\n",
 		      pid, ident_username);
-	      
 #if 0
 	    SIGNAL_HANDLE(SIGCHLD, SIG_DFL);
 #else
@@ -514,18 +533,21 @@ main(argc, argv)
 	    SIGNAL_HANDLE(SIGALRM, timedout);
 	    SIGNAL_HANDLE(SIGHUP,  SIG_IGN);
 	    SIGNAL_HANDLE(SIGTERM, SIG_DFL);
-	    smtpserver(1);
+	    smtpserver(1,&raddr);
 	  } else { /* Not from under the inetd -- standalone server */
 	    if (postoffice == NULL
 		&& (postoffice = getzenv("POSTOFFICE")) == NULL)
 	      postoffice = POSTOFFICE;
-	    if (!port_set)
+	    if (!port_set) {
 	      if (killprevious(SIGTERM, PID_SMTPSERVER) != 0) {
 		fprintf(stderr,
 			"%s: Can't write my pidfile!  Disk full ?\n",
 			progname);
 		exit(2);
 	      }
+	      fflush(stdout);
+	      fflush(stderr);
+	    }
 	    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	      fprintf(stderr,
 		      "%s: socket(AF_INET, SOCK_STREAM): %s\n",
@@ -539,6 +561,14 @@ main(argc, argv)
 		      progname, strerror(errno));
 	      exit(1);
 	    }
+#ifdef SO_REUSEPORT
+	    if (setsockopt(s, SOL_SOCKET, SO_REUSEPORT, (caddr_t)&i, sizeof i) < 0) {
+	      fprintf(stderr,
+		      "%s: setsockopt(SO_REUSEPORT): %s\n",
+		      progname, strerror(errno));
+	      exit(1);
+	    }
+#endif
 	    if (port <= 0) {
 	      struct servent *service;
 #ifdef	IPPORT_SMTP
@@ -559,7 +589,23 @@ main(argc, argv)
 		      progname, strerror(errno));
 	      exit(1);
 	    }
-	    if (listen(s, 20) < 0) {
+
+	    /* Set the listen limit HIGH; there has been an active
+	       denial-of-service attack where people send faked SYNs
+	       to open a connection to our system who does not want to
+	       talk with us at all -- or whose traffic goes thru a
+	       routing black-hole, who just plain don't exist on some
+	       existing network.  The net result being that our SYN-ACKs
+	       will never get ACKed, and no connection built...
+	       The SYN_RECV-queue can grow also due to routing diode
+	       behind of which some poor fellow is trying to get our
+	       attention, but our replies do not reach back to him/her. */
+	    /* Often the system enforces some upper bound for this
+	       limit, and you can't exceed it -- but some new systems
+	       allow rather high limits, lets try to use it!
+	       (The classical default is: 5) */
+
+	    if (listen(s, 2000) < 0) {
 	      fprintf(stderr, "%s: listen(): %s\n",
 		      progname, strerror(errno));
 	      exit(1);
@@ -579,7 +625,7 @@ main(argc, argv)
 #if 1
 	    pid = getpid();
 	    if (!logfp)
-	      openlogfp(1);
+	      openlogfp(daemon_flg);
 	    if (logfp != NULL) {
 	      time_t now;
 	      char *cp;
@@ -603,7 +649,8 @@ main(argc, argv)
 	      raddrlen = sizeof(raddr);
 	      msgfd = accept(s, (struct sockaddr *)&raddr, &raddrlen);
 	      if (msgfd < 0) {
-		switch (errno) {
+		int err = errno;
+		switch (err) {
 		  case EINTR:		/* very common.. */
 		      continue;
 		  case ECONNRESET:	/* seen to happen! */
@@ -614,15 +661,21 @@ main(argc, argv)
 		  case ECONNREFUSED:	/* unlikely.. */
 		  case EHOSTDOWN:
 		  case EHOSTUNREACH:
+#ifdef ENOSR
+		  case ENOSR:
+#endif
+#ifdef EPROTO
+		  case EPROTO:
+#endif
 		      continue;
 		  default:
 		      break;
 		}
 		fprintf(stderr,
-			"%s: accept(): %s\n", progname, strerror(errno));
+			"%s: accept(): %s\n", progname, strerror(err));
 		if (!logfp)
-		  openlogfp(1);
-		fprintf(logfp, "000000#\taccept(): %s\n", strerror(errno));
+		  openlogfp(daemon_flg);
+		fprintf(logfp, "000000#\taccept(): %s\n", strerror(err));
 		fflush(logfp);
 		exit(1);
 	      }
@@ -641,7 +694,8 @@ main(argc, argv)
 		  fclose(logfp);
 		logfp = NULL;
 
-		if (maxloadavg < loadavg_current()) {
+		if (maxloadavg != 999 &&
+		    maxloadavg < loadavg_current()) {
 		  write(msgfd, msg_toohighload,
 			strlen(msg_toohighload));
 		  exit(1);
@@ -652,7 +706,7 @@ main(argc, argv)
 		rport = raddr.sin_port;
 		setrhostname(&raddr);
 
-		openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
+		openlog("smtpserver", LOG_PID, LOG_MAIL);
 
 		if (ident_flag != 0)
 		  setrfc1413ident(msgfd,&raddr);
@@ -665,7 +719,7 @@ main(argc, argv)
 		pid = getpid();
 
 		if (!logfp)
-		  openlogfp(1);
+		  openlogfp(daemon_flg);
 		if (logfp != NULL)
 		  fprintf(logfp,
 			  "%d#\tconnection from %s\n", pid, ident_username);
@@ -678,7 +732,7 @@ main(argc, argv)
 		if (msgfd > 1)
 		  close(msgfd);
 
-		smtpserver(1);
+		smtpserver(1,&raddr);
 		if (routerpid > 0)
 		  killr(routerpid);
 		_exit(0);
@@ -977,10 +1031,9 @@ char *buf, *cp;
 	  return;
 	}
 	while (*cp == ' ' || *cp == '\t') ++cp;
-	if (sscanf(cp, "%s", helobuf) != 1) {
-	  type(501, NULL, "What is your domain name?");
-	  return;
-	}
+	*helobuf = 0;
+	sscanf(cp, "%s", helobuf);
+
 	/*
 	 * Craig P. says we have to spit back syntactically
 	 * invalid helo parameters at this stage, which is
@@ -1004,9 +1057,11 @@ char *buf, *cp;
 	/* At least check the input, though say "Ok, Master" while
 	   complaining.. */
 	if (!checkhelo && partridge(helobuf, cfinfo)) {
-	  rfc821_error_ptr += (cp - helobuf);
-	  type821err(-250, "", buf, "Invalid `%s' parameter!", buf);
-	  type(-250, NULL, "Err: %s", rfc821_error);
+	  if (carp->cmd != Hello) {
+	    rfc821_error_ptr += (cp - helobuf);
+	    type821err(-250, "", buf, "Invalid `%s' parameter!", buf);
+	    type(-250, NULL, "Err: %s", rfc821_error);
+	  }
 	  if (carp->cmd == Hello)
 	    strcpy(helobuf,"Bad.HELO.Input");
 	  else
@@ -1072,6 +1127,8 @@ int insecure;
 	char *drpt_ret;
 	char *bodytype = NULL;
 
+	sender_ok = 0; /* Set it, when we are sure.. */
+
 	/* For ESMTP SIZE-option use we need to know
 	   how much space we have, it is easiest by
 	   opening the spool file here, and asking
@@ -1259,6 +1316,22 @@ int insecure;
 	  }
 	}
 
+	reject_source = 0;
+	accept_sourcedomain = 1;
+	if (relaysourcedoms) {
+	  accept_sourcedomain = reltest(relaysourcedoms, cp);
+	}
+	if (!accept_sourcedomain && rejectsource) {
+	  reject_source = reltest(rejectsource, cp);
+	  if (reject_source) {
+	    type(553,"5.1.8","The source address is in a reject list");
+	    if (s) free(s);
+	    return;
+	  }
+	}
+	if (reject_source < 0)
+	  reject_source = 0;
+
 	if (mfp == NULL &&
 	    (mfp = mail_open(MSG_RFC822)) == NULL) {
 	  if (s)
@@ -1288,8 +1361,6 @@ int insecure;
 	if (drpt_ret != NULL)
 	  fprintf(mfp, "notaryret %s\n", drpt_ret);
 
-	sender_ok = 0; /* Set it, when we are sure.. */
-
 	if (ferror(mfp)) {
 	  type(452,m430,(char *)NULL);
 	  clearerr(mfp);
@@ -1762,11 +1833,84 @@ smtp_bdata(buf,cp)
 	return 0;
 }
 
+static void
+smtp_verify(buf,cp,cfinfo)
+     char *buf, *cp;
+     struct smtpconf *cfinfo;
+{
+	char *s;
+	int cfi;
+
+	if (state == Hello) {
+	  type(503, m551, "Waiting for HELO/EHLO command");
+	  return;
+	}
+	while (*cp == ' ' || *cp == '\t') ++cp;
+	if (*cp == '<')
+	  s = rfc821_path(cp,cfi); /* with < > */
+	else
+	  s = rfc821_path2(cp,cfi); /* Without < > */
+	if (s == cp) {
+	  type821err(501,m552, buf,"Path data: %s",rfc821_error);
+	  return;
+	}
+	while (*s == ' ' || *s == '\t') ++s;
+	if (*s != 0) {
+	  type(501, m552, "Growl! Extra junk after the VRFY argument!");
+	  return;
+	}
+	if ((cfi = STYLE(cfinfo, 'v'))) {
+	  if ((s = router(RKEY_VERIFY, 0, cp)) != NULL) {
+	    /* printf("%s\r\n", s);
+	       free(s); */
+	  } else
+	    type(501,m540, "Unable to verify that address");
+	} else
+	  type(252, "2.5.2", (char *)NULL); /* Syntax ok */
+}
+
+static void
+smtp_expand(buf,cp,cfinfo)
+     char *buf, *cp;
+     struct smtpconf *cfinfo;
+{
+	char *s;
+	int cfi;
+
+	if (state == Hello) {
+	  type(503, m551, "Waiting for HELO/EHLO command");
+	  return;
+	}
+	if ((cfi = STYLE(cfinfo, 'e'))) {
+	  while (*cp == ' ' || *cp == '\t') ++cp;
+	  if (*cp == '<')
+	    s = rfc821_path(cp,cfi); /* with < > */
+	  else
+	    s = rfc821_path2(cp,cfi); /* Without < > */
+	  if (s == cp) {
+	    type821err(501,m552,buf,"Path data: %s",rfc821_error);
+	    return;
+	  }
+	  while (*s == ' ' || *s == '\t') ++s;
+	  if (*s != 0) {
+	    type(501, m552, "Growl! Extra junk after the EXPN argument!");
+	    return;
+	  }
+	  if ((s = router(RKEY_EXPAND, 0, cp)) != NULL) {
+	    /* printf("%s\r\n", s);
+	       free(s); */
+	  } else
+	    type(501,m540, "Unable to verify that address");
+	} else
+	  type(502,m540, (char *)NULL);
+}
+
 /* The SMTP-server itself */
 
 static void
-smtpserver(insecure)
+smtpserver(insecure,raddr)
 	int insecure;
+	struct sockaddr_in *raddr;
 {
 	char *cp;
 	time_t now;
@@ -1791,6 +1935,9 @@ smtpserver(insecure)
 
 	s_setup(FILENO(stdin));
 
+	/* Lets figure-out who we are this time around -- we may be on
+	   a machine with multiple identities per multiple interfaces,
+	   or via virtual IP-numbers, or ... */
 	localsocksize = sizeof(localsock);
 	if (getsockname(s_inputfd,
 			(struct sockaddr *)&localsock,&localsocksize) == 0 &&
@@ -1802,8 +1949,15 @@ smtpserver(insecure)
 	    strcpy(myhostname,hostent->h_name);
 	}
 
+	if (rejectnets != NULL)
+	  reject_net = reltestaddr(rejectnets,raddr);
+	if (relaytargets != NULL &&
+	    relaysourcenets != NULL) {
+	  accept_sourcenet = reltestaddr(relaysourcenets,raddr);
+	}
+
 	/* re-opening the log ?? */
-	openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
+	openlog("smtpserver", LOG_PID, LOG_MAIL);
 
 #ifdef HAVE_TCPD_H /* TCP-Wrapper code */
 	if (wantconn(s_inputfd, "smtp-receiver") == 0) {
@@ -1817,8 +1971,13 @@ smtpserver(insecure)
 	}
 #endif
 
-	type(220, NULL, "%s ZMailer Server %s ESMTP%s ready at %s",
-	     myhostname, VersionNumb, ident_flag ? "+IDENT":"", cp);
+	if (reject_net) {
+	  type(-553, NULL, "%s - You are on our reject-IP-address -list, GO AWAY!",
+	       myhostname);
+	  type(553, NULL, "Ask HELP for our contact information.");
+	} else
+	  type(220, NULL, "%s ZMailer Server %s ESMTP%s ready at %s",
+	       myhostname, VersionNumb, ident_flag ? "+IDENT":"", cp);
 	fflush(stdout);
 
 	state = Hello;
@@ -1935,6 +2094,12 @@ smtpserver(insecure)
 	    fflush(stdout);
 	    continue;
 	  }
+	  if (reject_net && carp->cmd != Quit && carp->cmd != Help) {
+	    type(-553,NULL,"You are on our reject-IP-address -list, GO AWAY!");
+	    type(553, NULL, "Ask HELP for our contact information.");
+	    fflush(stdout);
+	    continue;
+	  }
 	  switch (carp->cmd) {
 	    case Null:
 		type(550,m550,"panic!");
@@ -1998,69 +2163,11 @@ smtpserver(insecure)
 		break;
 	    case Verify:
 	    case Verify2:
-		if (state == Hello) {
-		  type(503, m551, "Waiting for HELO/EHLO command");
-		  fflush(stdout);
-		  break;
-		}
-		{
-		  char *s;
-		  while (*cp == ' ' || *cp == '\t') ++cp;
-		  if (*cp == '<')
-		    s = rfc821_path(cp,cfi); /* with < > */
-		  else
-		    s = rfc821_path2(cp,cfi); /* Without < > */
-		  if (s == cp) {
-		    type821err(501,m552, buf,"Path data: %s",rfc821_error);
-		    break;
-		  }
-		  while (*s == ' ' || *s == '\t') ++s;
-		  if (*s != 0) {
-		    type(501, m552, "Growl! Extra junk after the VRFY argument!");
-		    break;
-		  }
-		}
-		if ((cfi = STYLE(cfinfo, 'v'))) {
-		  char *s;
-		  if ((s = router(RKEY_VERIFY, 0, cp)) != NULL) {
-		    printf("%s\r\n", s);
-		    free(s);
-		  } else
-		    type(501,m540, "Unable to verify that address");
-		} else
-		  type(252, "2.5.2", (char *)NULL); /* Syntax ok */
-
+		smtp_verify(buf,cp,cfinfo);
 		fflush(stdout);
 		break;
 	    case Expand:
-		if (state == Hello) {
-		  type(503, m551, "Waiting for HELO/EHLO command");
-		  fflush(stdout);
-		  break;
-		}
-		if ((cfi = STYLE(cfinfo, 'e'))) {
-		  char *s;
-		  while (*cp == ' ' || *cp == '\t') ++cp;
-		  if (*cp == '<')
-		    s = rfc821_path(cp,cfi); /* with < > */
-		  else
-		    s = rfc821_path2(cp,cfi); /* Without < > */
-		  if (s == cp) {
-		    type821err(501,m552,buf,"Path data: %s",rfc821_error);
-		    break;
-		  }
-		  while (*s == ' ' || *s == '\t') ++s;
-		  if (*s != 0) {
-		    type(501, m552, "Growl! Extra junk after the EXPN argument!");
-		    break;
-		  }
-		  if ((s = router(RKEY_EXPAND, 0, cp)) != NULL) {
-		    printf("%s\r\n", s);
-		    free(s);
-		  } else
-		    type(501,m540, "Unable to verify that address");
-		} else
-		  type(502,m540, (char *)NULL);
+		smtp_expand(buf,cp,cfinfo);
 		fflush(stdout);
 		break;
 	    case Turnme:
@@ -2083,6 +2190,7 @@ smtpserver(insecure)
 		VerboseCommand = 1;
 		break;
 	    case DebugMode:
+	        ++debug;
 		debug_report(VerboseCommand,rhostname,buf);
 		fflush(stdout);
 		break;
@@ -2252,6 +2360,12 @@ help(cfinfo, query)
 		TYPE_(Copyright);
 		TYPE_(Copyright2);
 		TYPE_("");
+		if (helplines[0] != NULL) {
+		  int i;
+		  for (i = 0; helplines[i] != NULL; ++i)
+		    TYPE_(helplines[i]);
+		  TYPE_("");
+		}
 		printf("214-The following commands are recognized:");
 		if (logfp)
 		  fprintf(logfp,"%dw\t214-The following commands are recognized:",pid);
@@ -2370,19 +2484,16 @@ char promptbuf[30];
 int  promptlen;
 FILE *tofp, *fromfp;
 
-char *
-mgets(buf, flagp, fp)
-	char *buf;	/* this is what I returned earlier, if non-null */
+extern char *mgets __((int *, FILE *));
+char *mgets(flagp, fp)
 	int *flagp;
 	FILE *fp;
 {
 	register int bufsize, c;
 	register char *cp;
-	char *s;
-	static char *obuf = NULL;
-	
-	if (buf != NULL)
-	  free(buf);
+	char *buf;
+	int ccnt = 0;
+
 	if (*flagp) {
 	  *flagp = 0;
 	  return NULL;
@@ -2392,6 +2503,7 @@ mgets(buf, flagp, fp)
 	cp = buf = emalloc(bufsize);
 	while ((c = getc(fp)) != EOF) {
 	  *cp = c;
+	  ++ccnt;
 	  if (c == '\n') {
 	    *flagp = 0;
 	    break;
@@ -2399,31 +2511,30 @@ mgets(buf, flagp, fp)
 	  ++cp;
 	  if (strncmp(cp - promptlen, promptbuf, promptlen) == 0) {
 	    free(buf);
+	    buf = NULL;
 	    *flagp = 1;
-	    break;
+	    return NULL;
 	  }
-	  if (bufsize - 2 < cp - buf) {
-	    bufsize *= 2;
-	    s = erealloc(buf, bufsize);
-	    cp  = s + (cp - buf);
-	    buf = s;
+	  if (bufsize - 2 < ccnt) {
+	    bufsize <<= 1;
+	    buf = erealloc(buf, bufsize);
+	    cp  = buf + ccnt;
 	  }
 	}
+
+	if (buf)
+	  *cp = '\0';
+
+	if (debug && buf)
+	  fprintf(stdout, "000 '%s'\r\n",buf);
+
 	if (c == EOF) {
 	  printf("got EOF!\n");
 	  free(buf);
 	  return NULL;
 	}
-	*cp = '\0';
-	if (obuf == NULL) {
-	  if (*flagp)
-	    return obuf;
-	  obuf = buf;
-	  return mgets((char *)NULL, flagp, fp);
-	}
-	cp = obuf;
-	obuf = *flagp ? NULL : buf;
-	return cp;
+
+	return buf;
 }
 
 char *newenviron[] = { "SMTPSERVER=y", (char *)0 };
@@ -2432,7 +2543,7 @@ int
 callr()
 {
 	int sawend, pid, to[2], from[2];
-	char *buf, *cp;
+	char *bufp, *cp;
 
 	if (pipe(to) < 0 || pipe(from) < 0)
 		return -1;
@@ -2445,12 +2556,12 @@ callr()
 	SIGNAL_HANDLE(SIGPIPE, SIG_IGN);
 
 	if (routerprog == NULL) {
-	  if ((buf = getzenv("MAILBIN")) == NULL) {
+	  if ((cp = getzenv("MAILBIN")) == NULL) {
 	    syslog(LOG_ERR, "MAILBIN unspecified in zmailer.conf");
 	    return -1;
 	  }
-	  routerprog = emalloc(strlen(buf)+sizeof "router"+2);
-	  sprintf(routerprog, "%s/router", buf);
+	  routerprog = emalloc(strlen(cp)+sizeof "router"+2);
+	  sprintf(routerprog, "%s/router", cp);
 	}
 
 	if ((pid = fork()) == 0) { /* child */
@@ -2492,15 +2603,15 @@ callr()
 		      ROUTER_SERVER, RKEY_INIT, rhostname, ihostaddr);
 	fflush(tofp);
 
-	buf = NULL;
 	sawend = 0;
-	while ((buf = mgets(buf, &sawend, fromfp)) != NULL) {
-	  if (strncmp(buf, BADEXEC, strlen(BADEXEC)-2) == 0) {
-	    free(buf);
+	while ((bufp = mgets(&sawend, fromfp)) != NULL) {
+	  if (strncmp(cp, BADEXEC, strlen(BADEXEC)-2) == 0) {
+	    free(cp);
 	    killr(pid);
 	    return -1;
 	  }
 	  /*printf("241%c%s\n", sawend == 1 ? ' ' : '-', buf);*/
+	  free(bufp);
 	}
 
 	return pid;
@@ -2527,8 +2638,8 @@ router(function, holdlast, args)
 	char *function, *args;
 	int holdlast;
 {
-	char *buf, *holdcp;
-	int sawend;
+	char *bufp, *prevb = NULL;
+	int sawend = 0;
 
 	if (args == NULL) {
 	  type(501,NULL,NULL);
@@ -2553,32 +2664,53 @@ router(function, holdlast, args)
 	fprintf(tofp, "\"\n");
 	fflush(tofp);
 
-	buf = holdcp = NULL;
-	sawend = 0;
-	while (!sawend && (buf = mgets(buf, &sawend, fromfp)) != NULL) {
+	for (;;) {
 	  /*
 	   * We want to give the router the opportunity to report
 	   * result codes, e.g. for boolean requests.  If the first
 	   * three characters are digits and the 4th a space or '-',
 	   * then pass through.
 	   */
-	  if (strlen(buf) > 4 &&
-	      isdigit(buf[0]) && isdigit(buf[1]) && isdigit(buf[2])
-	      && (buf[3] == ' ' || buf[3] == '-')) {
-	    if (holdlast && buf[3] == ' ')
-	      holdcp = buf;
-	    else
-	      printf("%s\r\n", buf);
-	  } else {
-	    if (holdlast && sawend)
-	      holdcp = buf;
-	    else
-	      printf("250%c%s\r\n",
-		     sawend == 1 ? ' ' : '-', buf);
+	  bufp = mgets(&sawend, fromfp);
+	  if (!bufp) break;
+
+	  if (debug) {
+	    fprintf(stdout,"001 Got string: '%s'\r\n", bufp);
+	    fflush(stdout);
 	  }
+
+	  if (prevb != NULL) {
+	    if (strlen(prevb) > 4 &&
+		isdigit(prevb[0]) && isdigit(prevb[1]) && isdigit(prevb[2])
+		&& (prevb[3] == ' ' || prevb[3] == '-')) {
+	      printf("%s\r\n", prevb);
+	    } else {
+	      printf("250-%s\r\n", prevb);
+	    }
+	    free(prevb);
+	  }
+	  prevb = bufp;
+	}
+
+	if (!holdlast) {
+	  /* -------- Print the last line too -------- */
+	  if (strlen(prevb) > 4 &&
+	      isdigit(prevb[0]) && isdigit(prevb[1]) && isdigit(prevb[2])
+	      && (prevb[3] == ' ')) {
+	    printf("%s\r\n", prevb);
+	  } else {
+	    printf("250 %s\r\n", prevb);
+	  }
+	  free(prevb);
+	  prevb = "<any-pointer>";
+	} else {
+	  /* Uhhh.... No output! */
+	  if (!prevb)
+	    printf("500 **INTERNAL*ERROR**\r\n");
 	}
 	fflush(stdout);
-	return holdcp;
+
+	return prevb; /* It may be unfreeable pointer... */
 }
 
 
@@ -2591,9 +2723,9 @@ type821err(code,status,inbuf,msg,s1,s2,s3)
 	char *s = inbuf+3+strlen(status)+1;
 	int maxcnt = 200;
 
-	printf("%03d-%s ", code < 0 ? -code : code,status);
+	printf("%03d-%s ", code < 0 ? -code : code, status);
 	if (logfp != NULL)
-	  fprintf(logfp, "%dw\t%03d-%s ", pid, code,status);
+	  fprintf(logfp, "%dw\t%03d-%s ", pid, code<0 ? -code : code, status);
 	while (s < rfc821_error_ptr && --maxcnt >= 0) {
 	  ++s;
 	  putc(' ',stdout);
@@ -3121,9 +3253,81 @@ struct sockaddr_in *raddr;
     }
 }
 
+
 /* as in: SKIPWHILE(isascii,cp) */
 #define SKIPWHILE(X,Y)  while (*Y != '\0' && isascii(*Y) && X(*Y)) { ++Y; }
 
+void
+cfparam(str)
+char *str;
+{
+  char *name, *param1, *param2;
+
+  name = strchr(str,'\n'); /* The trailing newline chopper ... */
+  if (name) *name = 0;
+
+  SKIPWHILE(!isspace,str);
+  SKIPWHILE(isspace,str);
+  name = str;
+  SKIPWHILE(!isspace,str);
+  if (*str != 0) *str++ = 0;
+
+  if (strcasecmp(name,"help")== 0) {
+    int i = 0, helpmax = (sizeof(helplines)/sizeof(char*)) -2;
+    while (helplines[i] != NULL && i < helpmax) ++i;
+    param2 = strchr(str,'\n');
+    if (param2) *param2 = 0;
+    helplines[i] = strdup(str);
+    helplines[i+1] = NULL; /* This will always stay within the array... */
+    return;
+  }
+
+  SKIPWHILE(isspace,str);
+  param1 = str;
+
+  SKIPWHILE(!isspace,str);
+  if (*str != 0) *str++ = 0;
+  SKIPWHILE(isspace,str);
+  param2 = str;
+  SKIPWHILE(!isspace,str);
+  if (*str != 0) *str++ = 0;
+
+  if (strcasecmp(name,"maxsize") == 0) {
+    sscanf(param1,"%d",maxsize);
+    return;
+  }
+
+  if (strcasecmp(name,"accept-percent-kludge") == 0) {
+    percent_accept = 1;
+    return;
+  }
+  if (strcasecmp(name,"reject-percent-kludge") == 0) {
+    percent_accept = -1;
+    return;
+  }
+  /* Following have two parameters:  DBTYPE and DBPATH */
+  if (strcasecmp(name,"relaytargets") == 0) {
+    reldefine(&relaytargets,param1,param2);
+    return;
+  }
+  if (strcasecmp(name,"relaycustnets") == 0) {
+    reldefine(&relaysourcenets,param1,param2);
+    return;
+  }
+  if (strcasecmp(name,"rejectnets") == 0) {
+    reldefine(&rejectnets,param1,param2);
+    return;
+  }
+  if (strcasecmp(name,"rejectsource") == 0) {
+    reldefine(&rejectsource,param1,param2);
+    return;
+  }
+  if (strcasecmp(name,"rejecttarget") == 0) {
+    reldefine(&rejecttarget,param1,param2);
+    return;
+  }
+}
+
 struct smtpconf *
 readcffile(name)
 	char *name;
@@ -3135,42 +3339,57 @@ readcffile(name)
 	if ((fp = fopen(name, "r")) == NULL)
 		return NULL;
 	head = NULL;
+	buf[sizeof(buf)-1] = 0;
 	while (fgets(buf, sizeof buf, fp) != NULL) {
-		if (buf[0] == '#' || (isascii(buf[0]) && isspace(buf[0])))
-			continue;
-		scf.flags = "";
-		scf.next = NULL;
-		cp = buf;
-		SKIPWHILE(!isspace,cp);
-		c = *cp;
-		*cp = '\0';
-		scf.pattern = strdup(buf);
-		scf.maxloadavg = 999;
-		if (c != '\0') {
-		  ++cp;
-		  SKIPWHILE(isspace,cp);
-		  if (*cp && isascii(*cp) && isdigit(*cp)) {
-		    /* Sanity-check -- 2 is VERY LOW */
-		    if ((scf.maxloadavg = atoi(cp)) < 2)
-		      scf.maxloadavg = 2;
-		    SKIPWHILE(isdigit,cp);
-		    SKIPWHILE(isspace,cp);
-		  }
-		  scf.flags = strdup(cp);
-		  if ((cp = strchr(scf.flags, '\n')) != NULL)
-		    *cp = '\0';
-		}
-		for (cp = scf.pattern; *cp != '\0'; ++cp)
-			if (isascii(*cp) && isalpha(*cp) && isupper(*cp))
-				*cp = tolower(*cp);
-		if (head == NULL) {
-			head = tail = (struct smtpconf *)emalloc(sizeof scf);
-			*head = scf;
-		} else {
-			tail->next = (struct smtpconf *)emalloc(sizeof scf);
-			*(tail->next) = scf;
-			tail = tail->next;
-		}
+	  c = buf[0];
+	  if (c == '#' || (isascii(c) && isspace(c)))
+	    continue;
+	  if (buf[sizeof(buf)-1] != 0 &&
+	      buf[sizeof(buf)-1] != '\n') {
+	    int cc;
+	    while ((cc = getc(fp)) != '\n' &&
+		   cc != EOF)
+	      ; /* Scan until end-of-line */
+	  }
+	  buf[sizeof(buf)-1] = 0; /* Trunc, just in case.. */
+
+	  if (strncmp(buf,"PARAM",5)==0) {
+	    cfparam(buf);
+	    continue;
+	  }
+	  scf.flags = "";
+	  scf.next = NULL;
+	  cp = buf;
+	  SKIPWHILE(!isspace,cp);
+	  c = *cp;
+	  *cp = '\0';
+	  scf.pattern = strdup(buf);
+	  scf.maxloadavg = 999;
+	  if (c != '\0') {
+	    ++cp;
+	    SKIPWHILE(isspace,cp);
+	    if (*cp && isascii(*cp) && isdigit(*cp)) {
+	      /* Sanity-check -- 2 is VERY LOW */
+	      if ((scf.maxloadavg = atoi(cp)) < 2)
+		scf.maxloadavg = 2;
+	      SKIPWHILE(isdigit,cp);
+	      SKIPWHILE(isspace,cp);
+	    }
+	    scf.flags = strdup(cp);
+	    if ((cp = strchr(scf.flags, '\n')) != NULL)
+	      *cp = '\0';
+	  }
+	  for (cp = scf.pattern; *cp != '\0'; ++cp)
+	    if (isascii(*cp) && isalpha(*cp) && isupper(*cp))
+	      *cp = tolower(*cp);
+	  if (head == NULL) {
+	    head = tail = (struct smtpconf *)emalloc(sizeof scf);
+	    *head = scf;
+	  } else {
+	    tail->next = (struct smtpconf *)emalloc(sizeof scf);
+	    *(tail->next) = scf;
+	    tail = tail->next;
+	  }
 	}
 	fclose(fp);
 	return head;
