@@ -136,8 +136,6 @@ extern int   ranny();
 #define	putc	fputc
 #endif	/* lint */
 
-FILE *verboselog = NULL; /* getrr() needs this .. */
-
 int
 main(argc, argv)
 	int argc;
@@ -160,6 +158,9 @@ main(argc, argv)
 	SIGNAL_HANDLESAVE(SIGTERM, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGTERM, wantout);
+	SIGNAL_HANDLESAVE(SIGQUIT, SIG_IGN, oldsig);
+	if (oldsig != SIG_IGN)
+	  SIGNAL_HANDLE(SIGQUIT, wantout);
 	SIGNAL_HANDLESAVE(SIGHUP, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGHUP, wantout);
@@ -460,7 +461,7 @@ hold_ns(s)
 	char *s;
 {
 	struct qtypes *qtp;
-	char *cp, host[BUFSIZ];
+	char *cp, host[1024]; /* 256 should be enough .. */
 
 	res_init();
 
@@ -481,8 +482,9 @@ hold_ns(s)
 		  progname, cp);
 	  return 1;		/* inconsistency with search_res.c, yell! */
 	}
-	strcpy(host, s);
-	switch (getrrtype(host, sizeof host, qtp->value, 2)) {
+	strncpy(host, s, sizeof(host));
+	host[sizeof(host)-1] = 0;
+	switch (getrrtype(host, sizeof host, qtp->value, 2, NULL)) {
 	case 0:
 		return 1;	/* negative reply */
 	case 1:
