@@ -408,7 +408,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    rp->desc = &d;
 	    rp->orcpt = NULL;
 	    rp->notify = NULL;
-	    rp->notifyflgs = 0;
+	    rp->notifyflgs = _DSN_NOTIFY_FAILURE; /* Default behaviour */
 	    d.recipients = rp;
 	    rp->status = EX_OK;
 	    rp->newmsgheader = NULL;
