@@ -3047,11 +3047,11 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	     * due to a bug; if that is the case by you,
 	     * change to return EX_TEMPFAIL iff hp->aa == 0.
 	     */
-	    sprintf(remotemsg, "smtp; 500 (Bind: no such domain: %s)", host);
+	    sprintf(remotemsg, "smtp; 500 (DNS: no such domain: %s)", host);
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOHOST;
 	  case SERVFAIL:
-	    sprintf(remotemsg, "smtp; 500 (Bind: server failure: %s)", host);
+	    sprintf(remotemsg, "smtp; 500 (DNS: server failure: %s)", host);
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_TEMPFAIL;
 #ifdef OLDJEEVES
@@ -3072,11 +3072,11 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 #endif
 	  case NOTIMP:
 	  case REFUSED:
-	    sprintf(remotemsg, "smtp; 500 (Bind: unsupported query: %s)", host);
+	    sprintf(remotemsg, "smtp; 500 (DNS: unsupported query: %s)", host);
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOPERM;
 	  }
-	  sprintf(remotemsg, "smtp; 500 (Bind: unknown error, MX info unavailable: %s)", host);
+	  sprintf(remotemsg, "smtp; 500 (DNS: unknown error, MX info unavailable: %s)", host);
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
@@ -3194,7 +3194,7 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	  ++n;			/* found one! */
 	}
 	if (n == 0) {/* MX's exist, but their WKS's show no TCP smtp service */
-	  sprintf(remotemsg, "smtp; 500 (Bind: MX host does not support SMTP: %s)", host);
+	  sprintf(remotemsg, "smtp; 500 (DNS: MX host does not support SMTP: %s)", host);
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
