@@ -32,7 +32,7 @@ extern int errno;	/* Not all systems declare it in <errno.h> ... */
 
 extern char *strchr();
 
-extern void create_dbase();
+extern void create_dbase __((FILE *, void *, int));
 
 void
 usage(prog,err,errno)
@@ -112,7 +112,7 @@ create_dbase(infile,dbf,typ)
 	    /* Point to begin of data after separating white space! */
 	  }
 
-
+	  rc = -2;
 #ifdef HAVE_NDBM_H
 	  if (typ == 1) {
 	    Ndatum Ndat, Nkey;
@@ -139,7 +139,7 @@ create_dbase(infile,dbf,typ)
 	    Bkey.data = t;
 	    Bkey.size = strlen(t)+1;
 	    Bdat.data = s;
-	    Bdat.size = strlen(t)+1;
+	    Bdat.size = strlen(s)+1;
 	    rc = (dbfile->put)(dbfile,&Bkey,&Bdat,R_NOOVERWRITE);
 	  }
 #endif
@@ -172,6 +172,10 @@ char *argv[];
 	char *dbtype;
 	void *dbf;
 
+	/* Usage: */
+	/*  makendbm dbtype database.name [infilename|-] */
+	/* argv[] 0    1          2           3		 */
+
 	if (argc < 3) usage(argv[0],"too few arguments",0);
 	if (argc > 4) usage(argv[0],"too many arguments",0);
 	dbasename = argv[2];
@@ -221,7 +225,9 @@ char *argv[];
 	}
 #endif
 #ifdef HAVE_DB_H
-	if (typ == 1) {
+	if (typ == 3) {
+	  dbasename = strcpy(malloc(strlen(dbasename)+8),dbasename);
+	  strcat(dbasename,".db"); /* ALWAYS append this */
 	  dbfile = dbopen(dbasename, O_RDWR|O_CREAT|O_TRUNC, 0644,
 			  DB_BTREE, NULL);
 	  dbf = dbfile;
